//+------------------------------------+
//| Bollinger Band Scalper       |
//+------------------------------------+
//
// After Bollinger Band is touched, EA waits until price crosses center MA,
// then takes a position towards the other Bollinger Band.  Exits upon reaching
// either band.  

// Example:  Price touches Upper Bollinger Band.  When price crosses
// Moving Average, a short position is taken.  Exits for profit when lower
// band is reached, or exits for loss when upper band is re-touched.  
// Long positions are taken after touching lower band and crossing MA.
// Exits reversed for Long position.

// variables declared here are GLOBAL in scope

#property copyright "Matthew Ebersviller"


// generic user input
extern double Lots=0.1;
extern double MaximumRisk=0.02;
extern double DecreaseFactor=3;
extern double deviations=2.0;
extern int    bandPeriod=20;
extern int    Slippage=2;
extern int    StopLoss=40;


//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//| (This code swiped from sample "Moving Average" EA that comes     |
//| MT4 installation.                                                |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
//---- select lot size
   lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
//---- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//---- return lot size
   if(lot<0.1) lot=0.1;
   return(lot);
  }


//+------------------------------------+
//| Custom init (usually empty on EAs) |
//|------------------------------------|
// Called ONCE when EA is added to chart
int init()
  {
   return(0);
  }


//+------------------------------------+
//| Custom deinit(usually empty on EAs)|
//+------------------------------------+
// Called ONCE when EA is removed from chart
int deinit()
  {
   return(0);
  }


//+------------------------------------+
//| EA main code                       |
//+------------------------------------+
// Called EACH TICK and possibly every Minute
// in the case that there have been no ticks

int start()
  {

   double p=Point;
   int      cnt=0;
   int      OrdersPerSymbol=0;
   int      status=0;
   int      ticket;
   int      total;

   double  bollingerMA;
   double  upperBand;
   double  lowerBand;

   // Error checking
   if(AccountFreeMargin()<(1000*LotsOptimized()))        {Print("-----NO MONEY"); return(0);}
   if(Bars<100)                               {Print("-----NO BARS "); return(0);}

   
   // Intial Moving average
   bollingerMA=iMA(Symbol(),0,bandPeriod,0,MODE_SMA,PRICE_CLOSE,1);
   
   // Bollinger Bands
   upperBand=iBands(Symbol(),0,bandPeriod,deviations,0,PRICE_CLOSE,MODE_UPPER,0);
   lowerBand=iBands(Symbol(),0,bandPeriod,deviations,0,PRICE_CLOSE,MODE_LOWER,0);


   total=OrdersTotal();
   if(total<1) 
     {
      // no opened orders identified
      if(AccountFreeMargin()<(1000*Lots))
        {
         Print("We have no money. Free Margin = ", AccountFreeMargin());
         return(0);  
        }
     // Change 'status'
     // 1: Upper band touched; looking for short at SMA cross
     // 2: Short entered; looking for exit at Bollinger Band touch
     // 3: Lower band touched; looking for long at SMA cross
     // 4: Long entered; looking for exit at Bollinger Band touch
     // 55:  Bogus status to prevent multiple entries
     if (High[0]>=upperBand) {
       status = 1;
     }
   
     if (Low[0]<=lowerBand) {
       status = 3;
     }

     if(status==1)
     {
       if (Low[0]<=bollingerMA)
       {
         status = 2;
       }
     }
    
     if(status==2)
     {
       ticket=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Ask,Slippage,Ask+(StopLoss*p),0,"SELL "+CurTime(),0,0,White);
       status = 55;
         if(ticket>0)
           {
            if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) Print("SELL order opened : ",OrderOpenPrice());
           }
         else Print("Error opening SELL order : ",GetLastError()); 
         return(0); 

     }
   }
      
      
      if(status==3)
     {
       if (High[0]>=bollingerMA)
       {
         status = 4;
       }
     }
      
     if(status==4)
     {
       ticket=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Bid,Slippage,Bid-(StopLoss*p),0,"BUY "+CurTime(),0,0,White);
       status = 55;
         if(ticket>0)
           {
            if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) Print("BUY order opened : ",OrderOpenPrice());
           }
         else Print("Error opening BUY order : ",GetLastError()); 
         return(0);
     }
           
//
//
// Close orders
//
//

// it is important to enter the market correctly, 
   // but it is more important to exit it correctly...   
   total=OrdersTotal();
   for(cnt=0;cnt<total;cnt++)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderType()<=OP_SELL &&   // check for opened position 
         OrderSymbol()==Symbol())  // check for symbol
        {
         if(OrderType()==OP_BUY)   // long position is opened
           {
            // should it be closed?
            if(High[0]>=upperBand)
                {
                 OrderClose(OrderTicket(),OrderLots(),Bid,3,Violet); // close position
                 return(0); // exit
                }
            // check for trailing stop
          else if(Low[0]<=lowerBand)
                {
                 OrderClose(OrderTicket(),OrderLots(),Bid,3,Violet); // close position
                }
           }

         else // go to short position
           {
            // should it be closed?
            if(Low[0]<=lowerBand)
              {
               OrderClose(OrderTicket(),OrderLots(),Ask,3,Violet); // close position
               return(0); // exit
              }
            else   if(High[0]>=lowerBand)
              {
               OrderClose(OrderTicket(),OrderLots(),Ask,3,Violet); // close position
               return(0); // exit
              }

           }
        }
     }
	

   return(0);
  } // start()